@@ -1,59 +1,67 @@
 bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
                           type = "bollen.stine", verbose = FALSE,
-                          return.LRT = FALSE, 
+                          return.LRT = FALSE,
                           double.bootstrap = "no",
-                          double.bootstrap.R = 500L, 
-                          double.bootstrap.alpha = 0.05, 
-                          warn = -1L, parallel = c("no", "multicore", "snow"), 
-                          ncpus = 1L, cl = NULL) 
+                          double.bootstrap.R = 500L,
+                          double.bootstrap.alpha = 0.05,
+                          warn = -1L,
+                          parallel = c("no", "multicore", "snow"),
+                          ncpus = 1L,
+                          cl = NULL,
+                          iseed = NULL)
 {
     # checks
     type <- tolower(type)
-    stopifnot(inherits(h0, "lavaan"), 
-              inherits(h1, "lavaan"), 
+    stopifnot(inherits(h0, "lavaan"),
+              inherits(h1, "lavaan"),
               type %in% c("bollen.stine", "parametric", "yuan", "nonparametric",
                           "ordinary"),
               double.bootstrap %in% c("no", "FDB", "standard"))
     if(type == "nonparametric") type <- "ordinary"
 
+    # check for conditional.x = TRUE
+    if(h0@Model@conditional.x) {
+        stop("lavaan ERROR: this function is not (yet) available if conditional.x = TRUE")
+    }
+
     old_options <- options(); options(warn = warn)
 
     # prepare
     LRT <- rep(as.numeric(NA), R)
-    if((h1@Fit@fx - h0@Fit@fx) > (.Machine$double.eps * 10)) { 
+    if((h1@optim$fx - h0@optim$fx) > sqrt(.Machine$double.eps)) {
         # restricted fit should not be better!
-        cat(" ... h0@Fit@fx = ", h0@Fit@fx, "h1@Fit@fx = ", h1@Fit@fx,
+        cat(" ... h0@optim$fx = ", h0@optim$fx, "h1@optim$fx = ", h1@optim$fx,
             "h0 should not be better!\n")
         options(old_options)
         return(NULL)
     }
     LRT.original <- abs(anova(h0, h1)$`Chisq diff`[2L])
     # abs only needed because df may be the same for both models!
-  
-  
+
+
     if(double.bootstrap == "FDB") {
         LRT.2 <- numeric(R)
     } else if(double.bootstrap == "standard") {
         plugin.pvalues <- numeric(R)
     }
-  
-  
+
+
     # prepare for parallel processing
     if(missing(parallel)) parallel <- "no"
     parallel <- match.arg(parallel)
     have_mc <- have_snow <- FALSE
     if(parallel != "no" && ncpus > 1L) {
-        if(parallel == "multicore") 
+        if(parallel == "multicore")
             have_mc <- .Platform$OS.type != "windows"
-        else if(parallel == "snow") 
+        else if(parallel == "snow")
             have_snow <- TRUE
-        if(!have_mc && !have_snow) 
+        if(!have_mc && !have_snow)
             ncpus <- 1L
     }
 
     #data
     data <- h0@Data
-  
+
     #Compute covariance matrix and additional mean vector
     if(type == "bollen.stine" || type == "parametric" || type == "yuan") {
         Sigma.hat <- computeSigmaHat(lavmodel = h0@Model)
@@ -69,7 +77,14 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
     } else {
         dataX <- data@X
     }
-  
+    lavdata <- h0@Data
+    lavoptions <- h0@Options
+    dataeXo <- lavdata@eXo
+    dataWT  <- lavdata@weights
+    dataMp  <- lavdata@Mp
+    dataRp  <- lavdata@Rp
+    dataLp  <- lavdata@Lp
+
     #Bollen-Stine data transformation
     if(type == "bollen.stine") {
         for(g in 1:h0@Data@ngroups) {
@@ -83,7 +98,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             X <- X %*% S.inv.sqrt %*% sigma.sqrt
 
             # add model based mean
-            if (h0@Model@meanstructure) 
+            if (h0@Model@meanstructure)
                 X <- scale(X, center = (-1 * Mu.hat[[g]]), scale = FALSE)
 
             # transformed data
@@ -93,7 +108,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
 
     # Yuan et al data transformation
     if(type == "yuan") {
-        # page numbers refer to Yuan et al, 2007      
+        # page numbers refer to Yuan et al, 2007
         # Define a function to find appropriate value of a
         # (p. 272)
         g.a <- function(a, Sigmahat, Sigmahat.inv, S, tau.hat, p){
@@ -106,7 +121,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
                                      (Sigmahat.inv - chol2inv(chol(S.a)))))
             res
         }
-      
+
         # Now use g.a within each group
         for(g in 1:h0@Data@ngroups) {
             S <- h0@SampleStats@cov[[g]]
@@ -139,7 +154,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             S.inv.sqrt <- lav_matrix_symmetric_sqrt(h0@SampleStats@icov[[g]])
 
             X <- data@X[[g]]
-            X <- X %*% S.inv.sqrt %*% S.a.sqrt            
+            X <- X %*% S.inv.sqrt %*% S.a.sqrt
 
             # transformed data
             dataX[[g]] <- X
@@ -152,33 +167,69 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             # take a bootstrap sample for each group
             for(g in 1:h0@Data@ngroups) {
                 stopifnot(h0@SampleStats@nobs[[g]] > 1L)
-                boot.idx <- sample(x = h0@SampleStats@nobs[[g]], 
+                boot.idx <- sample(x = h0@SampleStats@nobs[[g]],
                                    size = h0@SampleStats@nobs[[g]], replace = TRUE)
                 dataX[[g]] <- dataX[[g]][boot.idx,,drop=FALSE]
+                if(!is.null(dataeXo[[g]])) {
+                    dataeXo[[g]] <- dataeXo[[g]][boot.idx,,drop=FALSE]
+                }
+                if(!is.null(dataWT[[g]])) {
+                    dataWT[[g]] <- dataWT[[g]][boot.idx]
+                }
+                if(lavoptions$missing != "listwise") {
+                    dataMp[[g]] <- lav_data_missing_patterns(dataX[[g]],
+                                       sort.freq = FALSE, coverage = FALSE)
+                }
+                if(length(lavdata@ov.names.x[[g]]) == 0L &&
+                   all(lavdata@ov.names[[g]] %in%
+                       lavdata@ov$name[lavdata@ov$type == "ordered"])) {
+                    dataRp[[g]] <- lav_data_resp_patterns(dataX[[g]])
+                }
+                if(lavdata@nlevels > 1L) {
+                    # extract cluster variable(s), for this group
+                    clus <- matrix(0, nrow(dataX[[g]]), lavdata@nlevels - 1L)
+                    for(l in 2:lavdata@nlevels) {
+                        clus[,(l-1L)] <- lavdata@Lp[[g]]$cluster.idx[[l]]
+                    }
+                    dataLp[[g]] <- lav_data_cluster_patterns(Y = dataX[[g]],
+                                        clus = clus,
+                                        cluster = lavdata@cluster,
+                                        ov.names = lavdata@ov.names[[g]],
+                                        ov.names.l = lavdata@ov.names.l[[g]])
+                }
             }
         } else { # parametric!
             for(g in 1:h0@Data@ngroups) {
-                dataX[[g]] <- MASS::mvrnorm(n     = h0@SampleStats@nobs[[g]], 
-                                            mu    = Mu.hat[[g]], 
+                dataX[[g]] <- MASS::mvrnorm(n     = h0@SampleStats@nobs[[g]],
+                                            mu    = Mu.hat[[g]],
                                             Sigma = Sigma.hat[[g]])
             }
         }
 
+        # create new lavdata@ object (new in 0.6-2)
+        newData         <- lavdata
+        newData@X       <- dataX
+        newData@eXo     <- dataeXo
+        newData@weights <- dataWT
+        newData@Mp      <- dataMp
+        newData@Rp      <- dataRp
+        newData@Lp      <- dataLp
+
         # verbose
         if (verbose) cat("  ... bootstrap draw number: ", b, "\n")
 
         #Get sample statistics
         bootSampleStats <- try(lav_samplestats_from_data(
-                               lavdata  = NULL, 
-                               DataX    = dataX,
-                               DataOv        = data@ov,
-                               DataOvnames   = data@ov.names,
+                               lavdata  = newData,
                                missing       = h0@Options$missing,
-                               rescale  = (h0@Options$estimator == "ML" && 
-                                           h0@Options$likelihood =="normal"), 
+                               rescale  = (h0@Options$estimator == "ML" &&
+                                           h0@Options$likelihood =="normal"),
                                estimator     = h0@Options$estimator,
                                mimic         = h0@Options$mimic,
                                meanstructure = h0@Options$meanstructure,
+                               conditional.x = h0@Options$conditional.x,
+                               se            = h0@Options$se,
+                               test          = h0@Options$test,
                                group.w.free  = h0@Options$group.w.free,
                                missing.h1    = TRUE,
                                verbose  = FALSE), silent=TRUE)
@@ -188,55 +239,56 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             return(NULL)
         }
 
-        # just in case we need the new X in the data slot (lm!)
-        data@X <- dataX
-
         if (verbose) cat("  ... ... model h0: ")
         h0@Options$verbose <- FALSE
         h0@Options$se <- "none"
         h0@Options$test <- "standard"
+        h0@Options$baseline <- FALSE
+        h0@Options$h1 <- FALSE
 
         #Fit h0 model
         fit.h0 <- lavaan(slotOptions     = h0@Options,
-                         slotParTable    = h0@ParTable, 
-                         slotSampleStats = bootSampleStats, 
+                         slotParTable    = h0@ParTable,
+                         slotSampleStats = bootSampleStats,
                          slotData        = data)
-        if (!fit.h0@Fit@converged) {
+        if (!fit.h0@optim$converged) {
             if (verbose) cat("     FAILED: no convergence\n")
             options(old_options)
             return(NULL)
         }
-        if (verbose) 
-            cat("     ok -- niter = ", fit.h0@Fit@iterations, 
-                " fx = ", fit.h0@Fit@fx, "\n")
+        if (verbose)
+            cat("     ok -- niter = ", fit.h0@optim$iterations,
+                " fx = ", fit.h0@optim$fx, "\n")
 
         if (verbose) cat("  ... ... model h1: ")
         h1@Options$verbose <- FALSE
         h1@Options$se <- "none"
         h1@Options$test <- "standard"
+        h1@Options$baseline <- FALSE
+        h1@Options$h1 <- FALSE
 
         #Fit h1 model
-        fit.h1 <- lavaan(slotOptions     = h1@Options, 
-                         slotParTable    = h1@ParTable, 
-                         slotSampleStats = bootSampleStats, 
+        fit.h1 <- lavaan(slotOptions     = h1@Options,
+                         slotParTable    = h1@ParTable,
+                         slotSampleStats = bootSampleStats,
                          slotData        = data)
 
-        if (!fit.h1@Fit@converged) {
-            if (verbose) 
-                cat("     FAILED: no convergence -- niter = ", fit.h1@Fit@iterations, 
-                    " fx = ", fit.h1@Fit@fx,"\n")
+        if (!fit.h1@optim$converged) {
+            if (verbose)
+                cat("     FAILED: no convergence -- niter = ", fit.h1@optim$iterations,
+                    " fx = ", fit.h1@optim$fx,"\n")
             options(old_options)
             return(NULL)
         }
-        if (verbose) 
-            cat("     ok -- niter = ", fit.h1@Fit@iterations, 
-                " fx = ", fit.h1@Fit@fx, "\n")
+        if (verbose)
+            cat("     ok -- niter = ", fit.h1@optim$iterations,
+                " fx = ", fit.h1@optim$fx, "\n")
 
         # store LRT
-        if((fit.h1@Fit@fx - fit.h0@Fit@fx) > (.Machine$double.eps * 10)) {
-            #if((fit.h1@Fit@fx - fit.h0@Fit@fx) > 0.0) {
+        if((fit.h1@optim$fx - fit.h0@optim$fx) > sqrt(.Machine$double.eps)) {
+            #if((fit.h1@optim$fx - fit.h0@optim$fx) > 0.0) {
             if (verbose)
-                cat("  ... ... LRT  = <NA> h0 > h1, delta = ", fit.h1@Fit@fx - fit.h0@Fit@fx, "\n")
+                cat("  ... ... LRT  = <NA> h0 > h1, delta = ", fit.h1@optim$fx - fit.h0@optim$fx, "\n")
             options(old_options)
             return(NULL)
         } else {
@@ -244,42 +296,42 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             if (verbose)
                 cat("  ... ... LRT  = ", lrt.boot, "\n")
         }
-     
+
         #double bootstrap
         if (double.bootstrap == "standard") {
             if (verbose) cat("  ... ... calibrating p.value - ")
 
-            plugin.pvalue <- bootstrapLRT(h0 = fit.h0, h1 = fit.h1, 
-                                          R = double.bootstrap.R, 
-                                          type = type, 
-                                          verbose = FALSE, 
+            plugin.pvalue <- bootstrapLRT(h0 = fit.h0, h1 = fit.h1,
+                                          R = double.bootstrap.R,
+                                          type = type,
+                                          verbose = FALSE,
                                           return.LRT = FALSE, #FALSE
-                                          warn = warn, 
-                                          parallel = parallel, 
+                                          warn = warn,
+                                          parallel = parallel,
                                           ncpus = ncpus, cl = cl,
                                           double.bootstrap = "no")
             if (verbose) cat(sprintf("%5.3f", plugin.pvalue), "\n")
             attr(lrt.boot, "plugin.pvalue") <- plugin.pvalue
         } else if (double.bootstrap == "FDB") {
             #Fast double bootstrap
-            plugin.pvalue <- bootstrapLRT(h0 = fit.h0, h1 = fit.h1, 
-                                          R = 1L, 
-                                          type = type, 
-                                          verbose = FALSE, 
-                                          warn = warn, 
+            plugin.pvalue <- bootstrapLRT(h0 = fit.h0, h1 = fit.h1,
+                                          R = 1L,
+                                          type = type,
+                                          verbose = FALSE,
+                                          warn = warn,
                                           return.LRT = TRUE, #TRUE
-                                          parallel = parallel, 
+                                          parallel = parallel,
                                           ncpus = ncpus, cl = cl,
-                                          double.bootstrap = "no") 
+                                          double.bootstrap = "no")
 
             LRT.2 <- attr(plugin.pvalue, "LRT")
 
             if (verbose) cat("  ... ... LRT2 = ", LRT.2, "\n")
             attr(lrt.boot, "LRT.2") <- LRT.2
-        } 
+        }
         lrt.boot
     }
-  
+
     #Parallel processing
     RR <- sum(R)
     res <- if (ncpus > 1L && (have_mc || have_snow)) {
@@ -289,9 +341,9 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
         else if (have_snow) {
             if (is.null(cl)) {
                 cl <- parallel::makePSOCKcluster(rep("localhost", ncpus)) #
-                if (RNGkind()[1L] == "L'Ecuyer-CMRG") 
-                    parallel::clusterSetRNGStream(cl) # 
-                res <- parallel::parLapply(cl, seq_len(RR), fn) # 
+                if (RNGkind()[1L] == "L'Ecuyer-CMRG")
+                    parallel::clusterSetRNGStream(cl, iseed = iseed) #
+                res <- parallel::parLapply(cl, seq_len(RR), fn) #
                 parallel::stopCluster(cl) #
                 res
             }
@@ -303,7 +355,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
     for (b in seq_len(RR)) {
         if (!is.null(res[[b]])) {
             LRT[b] <- res[[b]]
-            if (double.bootstrap == "standard") { 
+            if (double.bootstrap == "standard") {
                 plugin.pvalues[b] <- attr(res[[b]], "plugin.pvalue")
             } else if (double.bootstrap == "FDB") {
                 LRT.2[b] <- attr(res[[b]], "LRT.2")
@@ -316,29 +368,29 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
 
     #Error handling
     if (length(error.idx) > 0L) {
-        warning("lavaan WARNING: only ", (R - length(error.idx)), 
+        warning("lavaan WARNING: only ", (R - length(error.idx)),
                 " bootstrap draws were successful")
         LRT <- LRT[-error.idx]
         if(length(LRT) == 0) LRT <- as.numeric(NA)
 
-        if (double.bootstrap == "standard") { 
+        if (double.bootstrap == "standard") {
             plugin.pvalues <- plugin.pvalues[-error.idx]
             attr(LRT, "error.idx") <- error.idx
 
         }
-        if (double.bootstrap == "FDB") { 
+        if (double.bootstrap == "FDB") {
             LRT.2 <- LRT.2[-error.idx]
             attr(LRT.2, "error.idx") <- error.idx
         }
     } else {
-        if (verbose) 
-            cat("Number of successful bootstrap draws:", (R - 
+        if (verbose)
+            cat("Number of successful bootstrap draws:", (R -
                                                           length(error.idx)), "\n")
     }
 
     pvalue <- sum(LRT > LRT.original) / length(LRT)
 
-    if (return.LRT) { 
+    if (return.LRT) {
         attr(pvalue, "LRT.original") <- LRT.original
         attr(pvalue, "LRT") <- LRT
     }
@@ -353,7 +405,7 @@ bootstrapLRT <- function (h0 = NULL, h1 = NULL, R = 1000L,
             attr(pvalue, "LRT.original") <- LRT.original
             attr(pvalue, "LRT") <- LRT
             attr(pvalue, "LRT2") <- LRT.2
-        }  
+        }
     } else if (double.bootstrap == "standard") {
         adj.alpha <- quantile(plugin.pvalues, double.bootstrap.alpha,
                               na.rm=TRUE)
