@@ -89,7 +89,7 @@ lavData <- function(data              = NULL,          # data.frame
             if(is.matrix(data)) {
                 if(nrow(data) == ncol(data)) {
                     # perhaps it is a covariance matrix?
-                    if(data[2,1] == data[1,2] && warn) { # not perfect...
+                    if(isSymmetric(data)) {
                         warning("lavaan WARNING: data argument looks like a covariance matrix; please use the sample.cov argument instead")
                     }
                 }
@@ -98,7 +98,8 @@ lavData <- function(data              = NULL,          # data.frame
                 ### data matrices directly
                 data <- as.data.frame(data, stringsAsFactors = FALSE)
             } else {
-                stop("lavaan ERROR: data object of class ", class(data))
+                stop("lavaan ERROR: data= argument",
+                     " is not a data.fame, but of class ", sQuote(class(data)))
             }
         }
 
@@ -636,6 +637,15 @@ lav_data_full <- function(data          = NULL,          # data.frame
         if(warn && any(f.names %in% unlist(ov.names.x)))
             warning(paste("lavaan WARNING: exogenous variable(s) declared as ordered in data:", paste(f.names, collapse=" ")))
     }
+    # check for ordered endogenous variables with more than 12 levels
+    if("ordered" %in% ov$type[!ov$name %in% unlist(ov.names.x)]) {
+        f.names <- ov$name[ov$type == "ordered" &
+                           !ov$name %in% unlist(ov.names.x) &
+                           ov$nlev > 12L]
+        if(warn && length(f.names) > 0L) {
+            warning(paste("lavaan WARNING: some ordered categorical variable(s) have more than 12 levels:", paste(f.names, collapse=" ")))
+        }
+    }
     # check for zero-cases
     idx <- which(ov$nobs == 0L | ov$var == 0)
     if(length(idx) > 0L) {
@@ -1234,107 +1244,104 @@ function(object) {
     lav_data_print_short(object)
 })
 
-lav_data_print_short <- function(object) {
+lav_data_print_short <- function(object, nd = 3L) {
 
     lavdata <- object
+    num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
 
     # listwise deletion?
-    listwise <- FALSE
+    listwise <- twocolumn <- FALSE
     for(g in 1:lavdata@ngroups) {
        if(lavdata@nobs[[1L]] != lavdata@norig[[1L]]) {
-           listwise <- TRUE
+           listwise <- twocolumn <- TRUE
            break
        }
     }
 
+    # header? no, for historical reasons only
     #cat("Data information:\n\n")
+
+    c1 <- c2 <- c3 <- character(0L)
+
+    # number of observations
     if(lavdata@ngroups == 1L) {
         if(listwise) {
-            cat(sprintf("  %-40s", ""), sprintf("  %10s", "Used"),
-                                        sprintf("  %10s", "Total"),
-                "\n", sep="")
+            c1 <- c(c1, ""); c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
         }
-        t0.txt <- sprintf("  %-40s", "Number of observations")
-        t1.txt <- sprintf("  %10i", lavdata@nobs[[1L]])
-        t2.txt <- ifelse(listwise,
-                  sprintf("  %10i", lavdata@norig[[1L]]), "")
-        cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        c1 <- c(c1, "Number of observations")
+        c2 <- c(c2, lavdata@nobs[[1L]])
+        c3 <- c(c3, ifelse(listwise, lavdata@norig[[1L]], ""))
+    } else {
+        c1 <- c(c1, "Number of observations per group:");
+        if(listwise) {
+            c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
+        } else {
+            c2 <- c(c2, ""); c3 <- c(c3, "")
+        }
+        for(g in 1:lavdata@ngroups) {
+            c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+            c2 <- c(c2, lavdata@nobs[[g]])
+            c3 <- c(c3, ifelse(listwise, lavdata@norig[[g]], ""))
+        } # g
+    }
 
+    # number of clusters
+    if(lavdata@ngroups == 1L) {
         if( (.hasSlot(lavdata, "nlevels")) && # in case we have an old obj
             (lavdata@nlevels > 1L) ) {
-            #cat("\n")
             for(l in 2:lavdata@nlevels) {
-                t0.txt <- sprintf("  %-40s",
-                    paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                          sep = ""))
-                t1.txt <- sprintf("  %10i", lavdata@Lp[[1]]$nclusters[[l]])
-                #t2.txt <- ifelse(listwise,
-                #          sprintf("  %10i", lavdata@norig[[1L]]), "")
-                t2.txt <- ""
-                cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+                c1 <- c(c1,
+                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
+                        sep = ""))
+                c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[l]])
+                c3 <- c(c3, "")
             }
         } else if( (.hasSlot(lavdata, "cluster")) &&
                    (length(lavdata@cluster) > 0L) ) {
-            t0.txt <- sprintf("  %-40s",
-            paste("Number of clusters [", lavdata@cluster, "]", sep = ""))
-            t1.txt <- sprintf("  %10i", lavdata@Lp[[1]]$nclusters[[2]])
-            t2.txt <- ""
-            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+            c1 <- c(c1, paste("Number of clusters [", lavdata@cluster, "]",
+                              sep = ""))
+            c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[2]])
+            c3 <- c(c3, "")
         }
     } else {
-        if(listwise) {
-            cat(sprintf("  %-40s", ""), sprintf("  %10s", "Used"),
-                                       sprintf("  %10s", "Total"),
-               "\n", sep="")
-        }
-        t0.txt <- sprintf("  %-40s", "Number of observations per group")
-        cat(t0.txt, "\n")
-        for(g in 1:lavdata@ngroups) {
-            t.txt <- sprintf("  %-40s  %10i", lavdata@group.label[[g]],
-                                              lavdata@nobs[[g]])
-            t2.txt <- ifelse(listwise,
-                      sprintf("  %10i", lavdata@norig[[g]]), "")
-            cat(t.txt, t2.txt, "\n", sep="")
-
-            if( (.hasSlot(lavdata, "nlevels")) &&
-                (lavdata@nlevels > 1L) ) {
-                #cat("\n")
-                for(l in 2:lavdata@nlevels) {
-                    t0.txt <- sprintf("  %-40s",
-                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                              sep = ""))
-                    t1.txt <- sprintf("  %10i",
-                                      lavdata@Lp[[g]]$nclusters[[l]])
-                    #t2.txt <- ifelse(listwise,
-                    #          sprintf("  %10i", lavdata@norig[[1L]]), "")
-                    t2.txt <- ""
-                    cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        if( (.hasSlot(lavdata, "nlevels")) && (lavdata@nlevels > 1L) ) {
+            for(l in 2:lavdata@nlevels) {
+                c1 <- c(c1,
+                  paste("Number of clusters [", lavdata@cluster[l-1], "]:",
+                        sep = ""))
+                c2 <- c(c2, ""); c3 <- c(c3, "")
+                for(g in 1:lavdata@ngroups) {
+                    c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                    c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[l]])
+                    c3 <- c(c3, "")
                 }
-            } else if( (.hasSlot(lavdata, "cluster")) &&
-                   (length(lavdata@cluster) > 0L) ) {
-                t0.txt <- sprintf("  %-40s",
-                paste("Number of clusters [", lavdata@cluster, "]", sep = ""))
-                t1.txt <- sprintf("  %10i", lavdata@Lp[[g]]$nclusters[[2]])
-                t2.txt <- ""
-                cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
             }
-        } # g
+        } else if( (.hasSlot(lavdata, "cluster")) &&
+               (length(lavdata@cluster) > 0L) ) {
+            c1 <- c(c1,
+             paste("Number of clusters [", lavdata@cluster, "]:", sep = ""))
+            c2 <- c(c2, ""); c3 <- c(c3, "")
+            for(g in 1:lavdata@ngroups) {
+                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[2]])
+                c3 <- c(c3, "")
+            }
+        }
     }
 
     # missing patterns?
     if(!is.null(lavdata@Mp[[1L]])) {
         if(lavdata@ngroups == 1L) {
-            t0.txt <- sprintf("  %-40s", "Number of missing patterns")
-            t1.txt <- sprintf("  %10i",
-                              lavdata@Mp[[1L]]$npatterns)
-            cat(t0.txt, t1.txt, "\n", sep="")
+            c1 <- c(c1, "Number of missing patterns")
+            c2 <- c(c2, lavdata@Mp[[1L]]$npatterns)
+            c3 <- c(c3, "")
         } else {
-            t0.txt <- sprintf("  %-40s", "Number of missing patterns per group")
-            cat(t0.txt, "\n")
+            c1 <- c(c1, "Number of missing patterns per group:")
+            c2 <- c(c2, ""); c3 <- c(c3, "")
             for(g in 1:lavdata@ngroups) {
-                t.txt <- sprintf("  %-40s  %10i", lavdata@group.label[[g]],
-                                 lavdata@Mp[[g]]$npatterns)
-                cat(t.txt, "\n", sep="")
+                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
+                c2 <- c(c2, lavdata@Mp[[g]]$npatterns)
+                c3 <- c(c3, "")
             }
         }
     }
@@ -1342,11 +1349,31 @@ lav_data_print_short <- function(object) {
     # sampling weights?
     if( (.hasSlot(lavdata, "weights")) && # in case we have an old object
         (!is.null(lavdata@weights[[1L]])) ) {
-        t0.txt <- sprintf("  %-30s", "Sampling weights variable")
-        t1.txt <- sprintf("  %20s", lavdata@sampling.weights)
-        cat(t0.txt, t1.txt, "\n", sep="")
+        c1 <- c(c1, "Sampling weights variable")
+        c2 <- c(c2, lavdata@sampling.weights)
+        c3 <- c(c3, "")
     }
 
-    cat("\n")
+    # empty row
+    c1 <- c(c1, ""); c2 <- c(c2, ""); c3 <- c(c3, "")
+
+    # format c1/c2
+    c1 <- format(c1, width = 43L)
+    c2 <- format(c2, width = 8L + max(0, (nd - 3L)) * 4L, justify = "right")
+    c3 <- format(c3, width = 8L + nd, justify = "right")
+
+    # create character matrix
+    if(twocolumn) {
+        M <- cbind(c1, c2, c3, deparse.level = 0)
+    } else {
+        M <- cbind(c1, c2, deparse.level = 0)
+    }
+    colnames(M) <- rep("",  ncol(M))
+    rownames(M) <- rep(" ", nrow(M))
+
+    # print
+    write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
+
+    invisible(M)
 }
 
