@@ -162,6 +162,7 @@ estimator.FIML <- function(Sigma.hat = NULL, Mu.hat = NULL, Yp = NULL,
         N <- sum(sapply(Yp, "[[", "freq"))
     }
 
+    # Note: we ignore x.idx (if any)
     fx <- lav_mvnorm_missing_loglik_samplestats(Yp = Yp,
                                                 Mu = Mu.hat, Sigma = Sigma.hat,
                                                 log2pi = FALSE,
@@ -194,6 +195,7 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
                           num.idx    = NULL,    # which variables are numeric
                           X          = NULL,    # raw data
                           eXo        = NULL,    # eXo data
+                          wt         = NULL,    # case weights
                           lavcache   = NULL,    # housekeeping stuff
                           missing    = NULL) {  # how to deal with missings?
 
@@ -453,32 +455,34 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
                 } else if(ov.types[i] == "numeric" &&
                           ov.types[j] == "ordered") {
                     # polyserial correlation
-                    logLIK <- ps_loglik_no_exo(Y1 = X[,i], Y2 = X[,j],
-                                               var.y1 = Sigma.hat[i,i],
-                                               eta.y1 = rep(Mu.hat[i], N),
-                                               th.y2 = TH[ th.idx == j ],
-                                               rho = Cor.hat[i,j])
+                    logLIK <- lav_bvmix_lik(Y1 = X[,i], Y2 = X[,j],
+                                            wt = wt,
+                                            evar.y1 = Sigma.hat[i,i],
+                                            beta.y1 = Mu.hat[i],
+                                            th.y2 = TH[ th.idx == j ],
+                                            rho = Cor.hat[i,j], .log = TRUE)
                     logLikPair[pstar.idx] <- sum(logLIK, na.rm = TRUE)
                 } else if(ov.types[j] == "numeric" &&
                           ov.types[i] == "ordered") {
                     # polyserial correlation
-                    logLIK <- ps_loglik_no_exo(Y1 = X[,j], Y2 = X[,i],
-                                               var.y1 = Sigma.hat[j,j],
-                                               eta.y1 = rep(Mu.hat[j], N),
-                                               th.y2 = TH[ th.idx == i ],
-                                               rho = Cor.hat[i,j])
+                    logLIK <- lav_bvmix_lik(Y1 = X[,j], Y2 = X[,i],
+                                            wt = wt,
+                                            evar.y1 = Sigma.hat[j,j],
+                                            beta.y1 = Mu.hat[j],
+                                            th.y2 = TH[ th.idx == i ],
+                                            rho = Cor.hat[i,j], .log = TRUE)
                     logLikPair[pstar.idx] <- sum(logLIK, na.rm = TRUE)
                 } else if(ov.types[i] == "ordered" &&
                           ov.types[j] == "ordered") {
                     # polychoric correlation
-                    pairwisePI <- pc_PI(rho   = Cor.hat[i,j],
-                                        th.y1 = TH[ th.idx == i ],
-                                        th.y2 = TH[ th.idx == j ])
+                    pairwisePI <- lav_bvord_noexo_pi(rho   = Cor.hat[i,j],
+                                                     th.y1 = TH[ th.idx == i ],
+                                                     th.y2 = TH[ th.idx == j ])
                     # avoid zeroes
                     pairwisePI[ pairwisePI < .Machine$double.eps] <-
                                              .Machine$double.eps
                     # note: missing values are just not counted
-                    FREQ <- pc_freq(X[,i], X[,j])
+                    FREQ <- lav_bvord_freq(X[,i], X[,j], wt = wt)
                     logLikPair[pstar.idx] <- sum(FREQ * log(pairwisePI))
                 }
             }
@@ -515,31 +519,38 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
                 if(ov.types[i] == "numeric" &&
                    ov.types[j] == "numeric") {
                     # ordinary pearson correlation
-                    LIK[,pstar.idx] <- pp_lik(Y1 = X[,i], Y2 = X[,j],
-                                              var.y1 = Sigma.hat[i,i],
-                                              eta.y1 = rep(Mu.hat[i], N),
-                                              eta.y2 = rep(Mu.hat[j], N),
-                                              var.y2 = Sigma.hat[j,j],
-                                              eXo = eXo,
-                                              rho = Cor.hat[i,j])
+                    LIK[,pstar.idx] <-
+                        lav_bvreg_lik(Y1 = X[,i], Y2 = X[,j], eXo = eXo,
+                                      wt      = wt,
+                                      evar.y1 = Sigma.hat[i,i],
+                                      beta.y1 = c(Mu.hat[i], PI[i,]),
+                                      evar.y2 = Sigma.hat[j,j],
+                                      beta.y2 = c(Mu.hat[j], PI[j,]),
+                                      rho     = Cor.hat[i,j])
                 } else if(ov.types[i] == "numeric" &&
                           ov.types[j] == "ordered") {
                     # polyserial correlation
                     ### FIXME: th.y2 should go into ps_lik!!!
-                    LIK[,pstar.idx] <- ps_lik(Y1 = X[,i], Y2 = X[,j],
-                                              var.y1 = Sigma.hat[i,i],
-                                              eta.y1 = rep(Mu.hat[i], N),
-                                              eXo = eXo,
-                                              rho = Cor.hat[i,j])
+                    LIK[,pstar.idx] <-
+                        lav_bvmix_lik(Y1 = X[,i], Y2 = X[,j], eXo = eXo,
+                                      wt = wt,
+                                      evar.y1 = Sigma.hat[i,i],
+                                      beta.y1 = c(Mu.hat[i], PI[i,]),
+                                      th.y2   = TH[th.idx==j],
+                                      sl.y2   = PI[j,],
+                                      rho     = Cor.hat[i,j])
                 } else if(ov.types[j] == "numeric" &&
                           ov.types[i] == "ordered") {
                     # polyserial correlation
                     ### FIXME: th.y1 should go into ps_lik!!!
-                    LIK[,pstar.idx] <- ps_lik(Y1 = X[,j], Y2 = X[,i],
-                                              var.y1 = Sigma.hat[j,j],
-                                              eta.y1 = rep(Mu.hat[j], N),
-                                              eXo = eXo,
-                                              rho = Cor.hat[i,j])
+                    LIK[,pstar.idx] <-
+                        lav_bvmix_lik(Y1 = X[,j], Y2 = X[,i], eXo = eXo,
+                                      wt = wt,
+                                      evar.y1 = Sigma.hat[j,j],
+                                      beta.y1 = c(Mu.hat[j], PI[j,]),
+                                      th.y2   = TH[th.idx==i],
+                                      sl.y2   = PI[i,],
+                                      rho = Cor.hat[i,j])
                 } else if(ov.types[i] == "ordered" &&
                           ov.types[j] == "ordered") {
                      LIK[,pstar.idx] <-
