@@ -238,7 +238,7 @@ lavaanify <- lavParTable <- function(
             block.rhs <- BLOCK.rhs[b]
             tmp <- try(scan(text = LIST[[block.lhs]], what = integer(),
                        quiet = TRUE), silent = TRUE)
-            if(class(tmp) == "integer") {
+            if(inherits(tmp, "integer")) {
                  LIST[[block.lhs]] <- tmp
             }
         }
@@ -415,6 +415,21 @@ lavaanify <- lavParTable <- function(
     # remove mod.idx column
     LIST$mod.idx <- NULL
 
+    # if lower/upper values were added, fix non-free values to ustart values
+    # new in 0.6-6
+    if(!is.null(LIST$lower)) {
+        fixed.idx <- which(LIST$free == 0L)
+        if(length(fixed.idx) > 0L) {
+            LIST$lower[fixed.idx] <- LIST$ustart[fixed.idx]
+        }
+    }
+    if(!is.null(LIST$upper)) {
+        fixed.idx <- which(LIST$free == 0L)
+        if(length(fixed.idx) > 0L) {
+            LIST$upper[fixed.idx] <- LIST$ustart[fixed.idx]
+        }
+    }
+
     if(debug) {
         cat("[lavaan DEBUG]: parameter LIST with MODIFIERS:\n")
         print( as.data.frame(LIST, stringsAsFactors=FALSE) )
@@ -437,6 +452,93 @@ lavaanify <- lavParTable <- function(
         print( as.data.frame(tmp, stringsAsFactors=FALSE) )
     }
 
+    # handle EFA equality constraints
+    # YR 14 Jan 2020: 0.6-6 does no longer impose 'explicit' constraints
+    #                 if we only need to fix a parameter to 0/1
+    # Note: we should also check if they are really needed:
+    #       eg., if all the factor-loadings of the 'second' set (time/group)
+    #       are constrained to be equal to the factor-loadings of the first
+    #       set, no further constraints are needed
+    if(auto.efa && !is.null(LIST$efa)) {
+        # for each set, for each block
+        nblocks <- lav_partable_nblocks(LIST)
+        set.names <- lav_partable_efa_values(LIST)
+        nsets <- length(set.names)
+
+        for(b in seq_len(nblocks)) {
+            for(s in seq_len(nsets)) {
+                # lv's for this block/set
+                lv.nam.efa <- unique(LIST$lhs[LIST$op == "=~" &
+                                              LIST$block == b &
+                                              LIST$efa == set.names[s]])
+                if(length(lv.nam.efa) == 1L) {
+                    # nothing to do (warn?)
+                    next
+                }
+
+                # equality constraints on ALL factor loadings in this set?
+                # --> no constraints are needed
+                #if(b > 1L || s > 1L) {
+                #    # collect plabels for this set, if any
+                #    set.idx <- which(LIST$op == "=~" &
+                #                     LIST$block == b &
+                #                     LIST$lhs %in% lv.nam.efa)
+                #    plabels.set <- LIST$plabel[ set.idx ]
+                #
+                #    # check simple == equalities
+                #    # TODO: what about more complex equality constraints?
+                #    eq.idx <- which(LIST$op == "==")
+                #    con.labels <- c(LIST$lhs[eq.idx],
+                #                    LIST$rhs[eq.idx])
+                #
+                #    # TODO: what about exceptions?
+                #    if(all(plabels.set %in% con.labels)) {
+                #        next
+                #    }
+                #}
+
+                # 1. echelon pattern
+                nfac <- length(lv.nam.efa)
+                for(f in seq_len(nfac)) {
+                    if(f == 1L) {
+                        next
+                    }
+                    nzero <- (f - 1L)
+                    ind.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa[f])
+                    if(length(ind.idx) < nzero) {
+                        stop("lavaan ERROR: efa factor ", lv.nam.efa[f],
+                             " has not enough indicators for echelon pattern")
+                    }
+
+                    # fix to zero
+                    LIST$free[  ind.idx[seq_len(nzero)]] <- 0L
+                    LIST$ustart[ind.idx[seq_len(nzero)]] <- 0
+                    LIST$user[  ind.idx[seq_len(nzero)]] <- 7L
+
+                }
+
+                # 2. covariances constrained to zero (only if oblique rotation)
+                if(!orthogonal.efa) {
+                    # skip if user == 1 (user-override!)
+                    cov.idx <- which(LIST$op == "~~" &
+                                     LIST$block == b &
+                                     LIST$user == 0L &
+                                     LIST$lhs %in% lv.nam.efa &
+                                     LIST$rhs %in% lv.nam.efa &
+                                     LIST$lhs != LIST$rhs)
+
+                    # fix to zero
+                    LIST$free[  cov.idx] <- 0L
+                    LIST$ustart[cov.idx] <- 0
+                    LIST$user[  cov.idx] <- 7L
+                }
+
+            } # sets
+        } # blocks
+    } # auto.efa
+
     # handle user-specified equality constraints
     # lavaan 0.5-18
     # - rewrite 'LABEL-based' equality constraints as == constraints
@@ -453,6 +555,24 @@ lavaanify <- lavParTable <- function(
             all.idx <- which(LABEL == eq.label) # all same-label parameters
             ref.idx <- all.idx[1L]              # the first one only
 
+            # new in 0.6-6: make sure lower/upper constraints are equal too
+            if(!is.null(LIST$lower) &&
+               length(unique(LIST$lower[all.idx])) > 0L) {
+                non.inf <- which(is.finite(LIST$lower[all.idx]))
+                if(length(non.inf) > 0L) {
+                    smallest.val <- min(LIST$lower[all.idx][non.inf])
+                    LIST$lower[all.idx] <- smallest.val
+                }
+            }
+            if(!is.null(LIST$upper) &&
+                length(unique(LIST$upper[all.idx])) > 0L) {
+                non.inf <- which(is.finite(LIST$upper[all.idx]))
+                if(length(non.inf) > 0L) {
+                    largest.val <- max(LIST$upper[all.idx][non.inf])
+                    LIST$upper[all.idx] <- largest.val
+                }
+            }
+
             # two possibilities:
             # 1. all.idx contains a fixed parameter: in this case,
             #    we fix them all (hopefully to the same value)
@@ -513,14 +633,6 @@ lavaanify <- lavParTable <- function(
     }
 
 
-    # count free parameters
-    idx.free <- which(LIST$free > 0)
-    LIST$free[idx.free] <- seq_along(idx.free)
-    # backwards compatibility...
-    if(!is.null(LIST$unco)) {
-         LIST$unco[idx.free] <- seq_along(idx.free)
-    }
-
 
     # handle constraints (if any) (NOT per group, but overall - 0.4-11)
     if(length(CON) > 0L) {
@@ -574,97 +686,6 @@ lavaanify <- lavParTable <- function(
     LIST$label[def.idx] <- LIST$lhs[def.idx]
 
 
-    # handle EFA equality constraints
-    # Note: we should also check if they are really needed:
-    #       eg., if all the factor-loadings of the 'second' set (time/group)
-    #       are constrained to be equal to the factor-loadings of the first
-    #       set, no further constraints are needed
-    if(auto.efa && !is.null(LIST$efa)) {
-        # for each set, for each block
-        nblocks <- lav_partable_nblocks(LIST)
-        set.names <- lav_partable_efa_values(LIST)
-        nsets <- length(set.names)
-
-        plabel <- character(0L)
-        for(b in seq_len(nblocks)) {
-            for(s in seq_len(nsets)) {
-                # lv's for this block/set
-                lv.nam.efa <- unique(LIST$lhs[LIST$op == "=~" &
-                                              LIST$block == b &
-                                              LIST$efa == set.names[s]])
-                if(length(lv.nam.efa) == 1L) {
-                    # nothing to do (warn?)
-                    next
-                }
-
-                # equality constraints on ALL factor loadings in this set?
-                # --> no constraints are needed
-                if(b > 1L || s > 1L) {
-                    # collect plabels for this set, if any
-                    set.idx <- which(LIST$op == "=~" &
-                                     LIST$block == b &
-                                     LIST$lhs %in% lv.nam.efa)
-                    plabels.set <- LIST$plabel[ set.idx ]
-
-                    # check simple == equalities
-                    # TODO: what about more complex equality constraints?
-                    eq.idx <- which(LIST$op == "==")
-                    con.labels <- c(LIST$lhs[eq.idx],
-                                    LIST$rhs[eq.idx])
-
-                    # TODO: what about exceptions?
-                    if(all(plabels.set %in% con.labels)) {
-                        next
-                    }
-                }
-
-                # 1. echelon pattern
-                nfac <- length(lv.nam.efa)
-                for(f in seq_len(nfac)) {
-                    if(f == 1L) {
-                        next
-                    }
-                    nzero <- (f - 1L)
-                    ind.idx <- which(LIST$op == "=~" &
-                                     LIST$block == b &
-                                     LIST$lhs %in% lv.nam.efa[f])
-                    if(length(ind.idx) < nzero) {
-                        stop("lavaan ERROR: efa factor ", lv.nam.efa[f],
-                             " has not enough indicators for echelon pattern")
-                    }
-
-                    # add these plabels
-                    plabel <- c(plabel, LIST$plabel[ind.idx[seq_len(nzero)]])
-                }
-
-                # 2. covariances constrained to zero (only if oblique rotation)
-                if(!orthogonal.efa) {
-                    # skip if user == 1 (user-override!)
-                    cov.idx <- which(LIST$op == "~~" &
-                                     LIST$block == b &
-                                     LIST$user == 0 &
-                                     LIST$lhs %in% lv.nam.efa &
-                                     LIST$rhs %in% lv.nam.efa &
-                                     LIST$lhs != LIST$rhs)
-                    plabel <- c(plabel, LIST$plabel[cov.idx])
-                }
-
-            } # sets
-        } # blocks
-
-        # add zero constraints
-        nel <- length(plabel)
-        TMP <- list()
-        TMP$lhs    <- plabel
-        TMP$op     <- rep("==", nel)
-        TMP$rhs    <- rep(0,    nel)
-        TMP$block  <- rep(0L,   nel)
-        TMP$user   <- rep(7L,   nel) # special code for EFA == 7
-        TMP$ustart <- rep(as.numeric(NA), nel)
-
-        LIST <- lav_partable_merge(LIST, TMP)
-    }
-
     # handle effect.coding related equality constraints
     if(is.logical(effect.coding) && effect.coding) {
         effect.coding <- c("loadings", "intercepts")
@@ -709,13 +730,13 @@ lavaanify <- lavParTable <- function(
                         # .p1. == 3 - .p2. - .p3.
                         # instead of
                         # 3 ==  .p1.+.p2.+.p3.
-                        # as this makes it easier to translate things to 
+                        # as this makes it easier to translate things to
                         # JAGS/stan
 
                         LHS <- plabel[1]
                         if(length(loadings.idx) > 1L) {
                             RHS <- paste(length(loadings.idx), "-",
-                                         paste(plabel, collapse = "-"), 
+                                         paste(plabel[-1], collapse = "-"),
                                          sep = "")
                         } else {
                             RHS <- length(loadings.idx)
@@ -745,7 +766,7 @@ lavaanify <- lavParTable <- function(
                         LHS <- plabel[1]
                         if(length(intercepts.idx) > 1L) {
                             RHS <- paste("0-",
-                                         paste(plabel, collapse = "-"), 
+                                         paste(plabel[-1], collapse = "-"),
                                          sep = "")
                         } else {
                             RHS <- 0L
@@ -765,7 +786,7 @@ lavaanify <- lavParTable <- function(
                         # free only if automatically added
                         if(length(lv.int.idx) > 0L &&
                            LIST$user[ lv.int.idx ] == 0L) {
-                            LIST$free[ lv.int.idx ] <- max(LIST$free) + 1L
+                            LIST$free[ lv.int.idx ] <- 1L
                         }
                     }
                 } # intercepts
@@ -781,7 +802,13 @@ lavaanify <- lavParTable <- function(
 
         TMP <- list()
 
-        lv.names <- unique(LIST$lhs[ LIST$op == "=~" ])
+        # do not include 'EFA' lv's
+        if(!is.null(LIST$efa)) {
+            lv.names <- unique(LIST$lhs[ LIST$op == "=~" &
+                                         !nchar(LIST$efa) > 0L ])
+        } else {
+            lv.names <- unique(LIST$lhs[ LIST$op == "=~" ])
+        }
         group.values <- lav_partable_group_values(LIST)
 
         for(lv in lv.names) {
@@ -797,9 +824,18 @@ lavaanify <- lavParTable <- function(
                 # 1) add eq constraint
                 plabel <- LIST$plabel[ lv.var.idx ]
 
-                TMP$lhs    <- c(TMP$lhs,  length(lv.var.idx))
-                TMP$op     <- c(TMP$op,   "==")
-                TMP$rhs    <- c(TMP$rhs,  paste(plabel, collapse = "+"))
+                LHS <- plabel[1]
+                if(length(lv.var.idx) > 1L) {
+                    RHS <- paste(length(lv.var.idx), "-",
+                                 paste(plabel[-1], collapse = "-"),
+                                 sep = "")
+                } else {
+                    RHS <- length(lv.var.idx)
+                }
+
+                TMP$lhs    <- c(TMP$lhs,    LHS)
+                TMP$op     <- c(TMP$op,     "==")
+                TMP$rhs    <- c(TMP$rhs,    RHS)
                 TMP$block  <- c(TMP$block,  0L)
                 TMP$user   <- c(TMP$user,   2L)
                 TMP$ustart <- c(TMP$ustart, as.numeric(NA))
@@ -813,7 +849,7 @@ lavaanify <- lavParTable <- function(
                 # free only if automatically added
                 if(length(lv.var.g1.idx) > 0L &&
                    LIST$user[ lv.var.g1.idx ] == 0L) {
-                    LIST$free[ lv.var.g1.idx ] <- max(LIST$free) + 1L
+                    LIST$free[ lv.var.g1.idx ] <- 1L
                 }
             }
         } # lv
@@ -821,6 +857,74 @@ lavaanify <- lavParTable <- function(
         LIST <- lav_partable_merge(LIST, TMP)
     }
 
+    # mg.lv.efa.variances
+    if(ngroups > 1L && "mg.lv.efa.variances" %in% effect.coding) {
+
+        TMP <- list()
+
+        # only 'EFA' lv's
+        if(!is.null(LIST$efa)) {
+            lv.names <- unique(LIST$lhs[ LIST$op == "=~" &
+                                         nchar(LIST$efa) > 0L ])
+        } else {
+            lv.names <- character(0L)
+        }
+        group.values <- lav_partable_group_values(LIST)
+
+        for(lv in lv.names) {
+            # factor variances
+            lv.var.idx <- which( LIST$op  == "~~" &
+                                 LIST$lhs == lv &
+                                 LIST$rhs == LIST$lhs &
+                                 LIST$lhs == lv )
+
+            # all free (but the first?)
+            if(length(lv.var.idx) > 0L &&
+               all(LIST$free[ lv.var.idx ][-1] > 0L)) {
+                # 1) add eq constraint
+                plabel <- LIST$plabel[ lv.var.idx ]
+
+                LHS <- plabel[1]
+                if(length(lv.var.idx) > 1L) {
+                    RHS <- paste(length(lv.var.idx), "-",
+                                 paste(plabel[-1], collapse = "-"),
+                                 sep = "")
+                } else {
+                    RHS <- length(lv.var.idx)
+                }
+
+                TMP$lhs    <- c(TMP$lhs,    LHS)
+                TMP$op     <- c(TMP$op,     "==")
+                TMP$rhs    <- c(TMP$rhs,    RHS)
+                TMP$block  <- c(TMP$block,  0L)
+                TMP$user   <- c(TMP$user,   2L)
+                TMP$ustart <- c(TMP$ustart, as.numeric(NA))
+
+                # 2) free lv variances first group
+                lv.var.g1.idx <- which( LIST$op  == "~~" &
+                                        LIST$group == group.values[1] &
+                                        LIST$lhs == lv &
+                                        LIST$rhs == LIST$lhs &
+                                        LIST$lhs == lv )
+                # free only if automatically added
+                if(length(lv.var.g1.idx) > 0L &&
+                   LIST$user[ lv.var.g1.idx ] == 0L) {
+                    LIST$free[ lv.var.g1.idx ] <- 1L
+                }
+            }
+        } # lv
+
+        LIST <- lav_partable_merge(LIST, TMP)
+    }
+
+
+    # count free parameters
+    idx.free <- which(LIST$free > 0)
+    LIST$free[idx.free] <- seq_along(idx.free)
+    # backwards compatibility...
+    if(!is.null(LIST$unco)) {
+         LIST$unco[idx.free] <- seq_along(idx.free)
+    }
 
     if(debug) {
         cat("[lavaan DEBUG] lavParTable\n")
