@@ -10,10 +10,10 @@ lavNames <- function(object, type = "ov", ...) {
 
     if(inherits(object, "lavaan") || inherits(object, "lavaanList")) {
          partable <- object@ParTable
-    } else if(class(object) == "list" ||
+    } else if(inherits(object, "list") ||
               inherits(object, "data.frame")) {
         partable <- object
-    } else if(class(object) == "character") {
+    } else if(inherits(object, "character")) {
         # just a model string?
         partable <- lavParseModelString(object)
     }
@@ -52,6 +52,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                    "ov.ind",      # observed indicators of latent variables
                    "ov.orphan",   # lonely observed intercepts/variances
                    "ov.interaction", # interaction terms (with colon)
+                   "ov.efa",      # indicators involved in efa
                    "th",          # thresholds ordinal only
                    "th.mean",     # thresholds ordinal + numeric variables
 
@@ -63,6 +64,9 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                    "lv.nox",      # non-exogenous latent variables
                    "lv.nonnormal",# latent variables with non-normal indicators
                    "lv.interaction", # interaction terms
+                   "lv.efa",      # latent variables involved in efa
+                   "lv.ind",      # latent indicators (higher-order cfa)
+                   "lv.marker",   # marker indicator per lv
 
                    "eqs.y",       # y's in regression
                    "eqs.x"        # x's in regression
@@ -80,8 +84,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
     # ALWAYS need `block' column -- create one if missing
     if(is.null(partable$block)) {
         partable$block <- rep(1L, length(partable$lhs))
-    } 
-   
+    }
+
     # nblocks -- block column is integer only
     nblocks <- lav_partable_nblocks(partable)
 
@@ -111,7 +115,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                     block.select <- ( block.select &
                                   partable[[block.var]] %in% block.val )
                 } else {
-                    stop("lavaan ERROR: selection variable `", 
+                    stop("lavaan ERROR: selection variable `",
                          block.var, " not found in the parameter table.")
                 }
 
@@ -120,7 +124,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                     stop("lavaan ERROR: ", block.var ,
                         " column does not contain value `", block.val, "'")
                 }
-                block.select <- ( block.select & 
+                block.select <- ( block.select &
+                                  !partable$op %in% c("==", "<", ">", ":=") &
                                   partable[[block.var]] %in% block.val )
             }
         } # dot
@@ -145,6 +150,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
     OUT$ov.ind         <- vector("list", length = nblocks)
     OUT$ov.orphan      <- vector("list", length = nblocks)
     OUT$ov.interaction <- vector("list", length = nblocks)
+    OUT$ov.efa         <- vector("list", length = nblocks)
+
     OUT$th             <- vector("list", length = nblocks)
     OUT$th.mean        <- vector("list", length = nblocks)
 
@@ -156,6 +163,9 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
     OUT$lv.nox         <- vector("list", length = nblocks)
     OUT$lv.nonnormal   <- vector("list", length = nblocks)
     OUT$lv.interaction <- vector("list", length = nblocks)
+    OUT$lv.efa         <- vector("list", length = nblocks)
+    OUT$lv.ind         <- vector("list", length = nblocks)
+    OUT$lv.marker      <- vector("list", length = nblocks)
 
     OUT$eqs.y          <- vector("list", length = nblocks)
     OUT$eqs.x          <- vector("list", length = nblocks)
@@ -204,6 +214,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.regular[[b]] <- out
         }
 
+
         # interaction terms involving latent variables (only)
         if("lv.interaction" %in% type) {
             OUT$lv.interaction[[b]] <- lv.interaction
@@ -216,6 +227,27 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.formative[[b]] <- out
         }
 
+        # lv's involved in efa
+        if(any(type %in% c("lv.efa", "ov.efa"))) {
+            if(is.null(partable$efa)) {
+                out <- character(0L)
+            } else {
+                set.names <- lav_partable_efa_values(partable)
+                out <- unique( partable$lhs[ partable$op == "=~" &
+                                             partable$block == b &
+                                             partable$efa %in% set.names ] )
+            }
+            OUT$lv.efa[[b]] <- out
+        }
+
+        # lv's that are indicators of a higher-order factor
+        if("lv.ind" %in% type) {
+            out <- unique( partable$rhs[ partable$block == b &
+                                         partable$op == "=~" &
+                                         partable$rhs %in% lv.names  ] )
+            OUT$lv.ind[[b]] <- out
+        }
+
         # eqs.y
         if(!(length(type) == 1L &&
            type %in% c("lv", "lv.regular", "lv.nonnormal"))) {
@@ -281,8 +313,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
 
             ov.tmp <- c(ov.ind, ov.y, ov.x)
             ov.extra <- unique(c(ov.cov, ov.int)) # must be in this order!
-                                                  # so that 
-                                                  # lav_partable_independence 
+                                                  # so that
+                                                  # lav_partable_independence
                                                   # retains the same order
             ov.names <- c(ov.tmp, ov.extra[ !ov.extra %in% ov.tmp ])
         }
@@ -322,6 +354,14 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$ov.interaction[[b]] <- ov.interaction
         }
 
+        if("ov.efa" %in% type) {
+            ov.efa <- partable$rhs[ partable$op == "=~" &
+                                    partable$block == b &
+                                    partable$rhs %in% ov.ind &
+                                    partable$lhs %in% OUT$lv.efa[[b]] ]
+            OUT$ov.efa[[b]] <- unique(ov.efa)
+        }
+
 
         # exogenous `x' covariates
         if(any(type %in% c("ov.x","ov.nox", "ov.model",
@@ -535,14 +575,42 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.nox[[b]] <- lv.names[! lv.names %in% lv.names.x ]
         }
 
-    }
+        # marker indicator (if any) for each lv
+        if("lv.marker" %in% type) {
+            # default: "" per lv
+            out <- character( length(lv.names) )
+            names(out) <- lv.names
+            for(l in seq_len( length(lv.names) )) {
+                this.lv.name <- lv.names[l]
+                # try to see if we can find a 'marker' indicator for this factor
+                marker.idx <- which(partable$block == b &
+                                    partable$lhs == this.lv.name &
+                                    partable$rhs %in% v.ind &
+                                    partable$free == 0L)
+                                    # no check for '1' (eg growth factor 0,1,2,)
+                if(length(marker.idx) == 1L) { # unique only!!
+                    out[l] <- partable$rhs[marker.idx]
+                }
+            }
+            OUT$lv.marker[[b]] <- out
+        }
+
+    } # b
 
     # to mimic old behaviour, if length(type) == 1L
     if(length(type) == 1L) {
         OUT <- OUT[[type]]
         # to mimic old behaviour, if specific block is requested
         if(ndotdotdot == 0L) {
-            OUT <- unique(unlist(OUT))
+            if(type == "lv.marker") {
+                OUT <- unlist(OUT)
+                # no unique, as unique drops attributes, and reduces
+                # c("", "", "") to a single ""
+                # (but, say for 2 groups, you get 2 copies)
+                # as this is only for 'display', we leave like that
+            } else {
+                OUT <- unique(unlist(OUT))
+            }
         } else if(length(block.select) == 1L) {
             OUT <- OUT[[block.select]]
         } else {
