@@ -5,7 +5,7 @@
 ##
 ## perhaps we should add 0.0005 or something to avoid this?
 
-print.lavaan.data.frame <- function(x, ..., nd=3) {
+print.lavaan.data.frame <- function(x, ..., nd = 3L) {
 
     ROW.NAMES <- rownames(x)
     y <- as.data.frame(lapply(x, function(x) {
@@ -17,6 +17,11 @@ print.lavaan.data.frame <- function(x, ..., nd=3) {
     }
 
     print(y, ...)
+
+    if(!is.null(attr(x, "footer"))) {
+        cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
@@ -40,7 +45,7 @@ print.lavaan.list <- function(x, ...) {
 
 
 # prints only lower triangle of a symmetric matrix
-print.lavaan.matrix.symmetric <- function(x, ..., nd=3) {
+print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L) {
     # print only lower triangle of a symmetric matrix
     # this function was inspired by the `print.correlation' function
     # in package nlme
@@ -48,23 +53,23 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd=3) {
     ll <- lower.tri(x, diag=TRUE)
     y[ll] <- format(round(x[ll], digits=nd)); y[!ll] <- ""
     if (!is.null(colnames(x))) {
-      colnames(y) <- abbreviate(colnames(x), minlength = nd + 3)
+      colnames(y) <- abbreviate(colnames(x), minlength = nd + 3L)
     }
     print(y, ..., quote = FALSE)
     invisible(x)
 }
 
 
-print.lavaan.matrix <- function(x, ..., nd=3) {
+print.lavaan.matrix <- function(x, ..., nd = 3L) {
     y <- unclass(x)
     if (!is.null(colnames(x))) {
-      colnames(y) <- abbreviate(colnames(x), minlength = nd + 3)
+      colnames(y) <- abbreviate(colnames(x), minlength = nd + 3L)
     }
     print( round(y, nd), ... )
     invisible(x)
 }
 
-print.lavaan.vector <- function(x, ..., nd=3) {
+print.lavaan.vector <- function(x, ..., nd = 3L) {
     y <- unclass(x)
     #if(!is.null(names(x))) {
     #    names(y) <- abbreviate(names(x), minlength = nd + 3)
@@ -81,8 +86,8 @@ print.lavaan.character <- function(x) {
 print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
 
     # format for numeric values
-    num.format  <- paste("%", max(8, nd + 5), ".", nd, "f", sep = "")
-    char.format <- paste("%", max(8, nd + 5), "s", sep="")
+    num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
+    char.format <- paste("%", max(8L, nd + 5L), "s", sep = "")
 
     # output sections
     GSECTIONS <- c("Latent Variables",
@@ -100,6 +105,9 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
 
     # header?
     header <- attr(x, "header")
+    if(is.null(header)) {
+        header <- FALSE
+    }
 
     if(header) {
         cat("\nParameter Estimates:\n\n")
@@ -110,55 +118,96 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         # 3. bootstrap requested/successful draws
         if(!is.null(x$se)) {
 
-            if(attr(x, "se") != "bootstrap") {
-                # 1.
-                t0.txt <- sprintf("  %-35s", "Information")
-                tmp.txt <- attr(x, "information")
-                t1.txt <- sprintf("  %15s",
-                                  paste(toupper(substring(tmp.txt,1,1)),
-                                  substring(tmp.txt,2), sep=""))
-                cat(t0.txt, t1.txt, "\n", sep="")
+            # container
+            c1 <- c2 <- character(0L)
 
-                # 2.
+            # which type of standard errors?
+            c1 <- c(c1, "Standard errors")
+            if(attr(x, "se") == "robust.huber.white") {
+                tmp.txt <- "sandwich" # since 0.6-6
+            } else {
+                tmp.txt <- attr(x, "se")
+            }
+            c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                                      substring(tmp.txt, 2), sep = ""))
+            # information
+            if(attr(x, "se") != "bootstrap") {
+
+                # type for information
+                if(attr(x, "se") == "robust.huber.white") {
+                    c1 <- c(c1, "Information bread")
+                } else {
+                    c1 <- c(c1, "Information")
+                }
+                tmp.txt <- attr(x, "information")
+                c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                                  substring(tmp.txt, 2), sep = ""))
+
+                # if observed, which type? (hessian of h1)
+                if(attr(x, "information") == "observed") {
+                    c1 <- c(c1, "Observed information based on")
+                    tmp.txt <- attr(x, "observed.information")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt, 1, 1)),
+                                      substring(tmp.txt, 2), sep = ""))
+                }
+
+                # if h1 is involved, structured or unstructured?
                 if(attr(x, "information") %in% c("expected", "first.order") ||
                    attr(x, "observed.information") == "h1") {
-                    t0.txt <- sprintf("  %-35s",
-                                      "Information saturated (h1) model")
+                    if(attr(x, "se") == "robust.huber.white" &&
+                       attr(x, "h1.information") !=
+                       attr(x, "h1.information.meat")) {
+                        c1 <- c(c1, "Information bread saturated (h1) model")
+                    } else {
+                        c1 <- c(c1, "Information saturated (h1) model")
+                    }
                     tmp.txt <- attr(x, "h1.information")
-                    t1.txt <- sprintf("  %15s",
-                                      paste(toupper(substring(tmp.txt,1,1)),
-                                            substring(tmp.txt,2), sep=""))
-                    cat(t0.txt, t1.txt, "\n", sep="")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt, 2), sep = ""))
                 }
-                if(attr(x, "information") == "observed") {
-                    t0.txt <- sprintf("  %-35s",
-                                      "Observed information based on")
-                    tmp.txt <- attr(x, "observed.information")
-                    t1.txt <- sprintf("  %15s",
-                                      paste(toupper(substring(tmp.txt,1,1)),
-                                      substring(tmp.txt,2), sep=""))
-                    cat(t0.txt, t1.txt, "\n", sep="")
+
+                # if sandwich, which information for the meat? (first.order)
+                # only print if it is NOT first.order
+                if(attr(x, "se") == "robust.huber.white" &&
+                   attr(x, "information.meat") != "first.order") {
+                    c1 <- c(c1, "Information meat")
+                    tmp.txt <- attr(x, "information.meat")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt, 2), sep = ""))
+                }
+
+                # if sandwich, structured or unstructured for the meat?
+                # only print if it is not the same as h1.information
+                if(attr(x, "se") == "robust.huber.white" &&
+                   attr(x, "h1.information.meat") !=
+                   attr(x, "h1.information")) {
+                    c1 <- c(c1, "Information meat saturated (h1) model")
+                    tmp.txt <- attr(x, "h1.information.meat")
+                    c2 <- c(c2, paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt, 2), sep = ""))
                 }
             } # no bootstrap
 
-            # 3.
-            t0.txt <- sprintf("  %-31s", "Standard Errors")
-            tmp.txt <- attr(x, "se")
-            t1.txt <- sprintf("  %19s", paste(toupper(substring(tmp.txt,1,1)),
-                                              substring(tmp.txt,2), sep=""))
-            cat(t0.txt, t1.txt, "\n", sep="")
-
             # 4.
             if(attr(x, "se") == "bootstrap" && !is.null(attr(x, "bootstrap"))) {
-                t0.txt <-
-                    sprintf("  %-40s", "Number of requested bootstrap draws")
-                t1.txt <- sprintf("  %10i", attr(x, "bootstrap"))
-                cat(t0.txt, t1.txt, "\n", sep="")
-                t0.txt <-
-                    sprintf("  %-40s", "Number of successful bootstrap draws")
-                t1.txt <- sprintf("  %10i", attr(x, "bootstrap.successful"))
-                cat(t0.txt, t1.txt, "\n", sep="")
+                c1 <- c(c1, "Number of requested bootstrap draws")
+                c2 <- c(c2, attr(x, "bootstrap"))
+                c1 <- c(c1, "Number of successful bootstrap draws")
+                c2 <- c(c2, attr(x, "bootstrap.successful"))
             }
+
+            # format c1/c2
+            c1 <- format(c1, width = 38L)
+            c2 <- format(c2,
+                   width = 13L + max(0, (nd - 3L)) * 4L, justify = "right")
+
+            # create character matrix
+            M <- cbind(c1, c2, deparse.level = 0)
+            colnames(M) <- rep("",  ncol(M))
+            rownames(M) <- rep(" ", nrow(M))
+
+            # print
+            write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
         }
     }
 
@@ -251,6 +300,39 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         }
     }
 
+    # handle lower/upper boundary points
+    if(!is.null(x$lower)) {
+        b.idx <- which( abs(x$lower - x$est) < sqrt(.Machine$double.eps) &
+                        is.finite(x$se) & x$se != 0.0)
+        if(length(b.idx) > 0L && !is.null(x$pvalue)) {
+            m[b.idx, "pvalue"] <- ""
+            if(is.null(x$label)) {
+                x$label <- rep("", length(x$lhs))
+            }
+            x$label[b.idx] <- ifelse(nchar(x$label[b.idx]) > 0L,
+                                     paste(x$label[b.idx], "+lb", sep = ""),
+                                     "lb")
+        }
+        # remove lower column
+        m <- m[, colnames(m) != "lower"]
+    }
+    if(!is.null(x$upper)) {
+        b.idx <- which( abs(x$upper - x$est) < sqrt(.Machine$double.eps) &
+                        is.finite(x$se) & x$se != 0.0)
+        if(length(b.idx) > 0L && !is.null(x$pvalue)) {
+            m[b.idx, "pvalue"] <- ""
+            if(is.null(x$label)) {
+                x$label <- rep("", length(x$lhs))
+            }
+            x$label[b.idx] <- ifelse(nchar(x$label[b.idx]) > 0L,
+                                     paste(x$label[b.idx], "+ub", sep = ""),
+                                     "ub")
+        }
+        # remove upper column
+        m <- m[, colnames(m) != "upper"]
+    }
+
+
     # handle fmi
     if(!is.null(x$fmi)) {
         se.idx <- which(x$se == 0)
@@ -655,14 +737,14 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
 
             # fit measures?
             b.options <- lavInspect(y$MM.FIT[[b]], "options")
-            if(b.options$test != "none") {
+            if(!(length(b.options$test) == 1L && b.options$test == "none")) {
                 cat("\n")
                 print(fitMeasures(y$MM.FIT[[b]], c("chisq", "df", "pvalue", "cfi", "rmsea", "srmr")))
             }
 
             # parameter estimates
-            PE <- parameterEstimates(y$MM.FIT[[b]], add.attributes = TRUE,
-                                     ci = FALSE)
+            PE <- parameterEstimates(y$MM.FIT[[b]], ci = FALSE,
+                                     output = "text", header = TRUE)
             print.lavaan.parameterEstimates(PE, ..., nd = nd)
             cat("\n")
         }
@@ -674,17 +756,17 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
         cat("\n")
         #print.lavaan.parameterEstimates(y$PE, ..., nd = nd)
 
-        short.summary(y$STRUC.FIT)
+        lav_object_print_short_summary(y$STRUC.FIT)
         FIT <- fitMeasures(y$STRUC.FIT, fit.measures="default")
         if(FIT["df"] > 0) {
-            print.fit.measures( FIT )
+            print.lavaan.fitMeasures( FIT, add.h0 = FALSE )
         }
     }
     PE <- parameterEstimates(y$STRUC.FIT, ci = FALSE,
                              remove.eq = FALSE, remove.system.eq = TRUE,
                              remove.ineq = FALSE, remove.def = FALSE,
                              remove.nonfree = FALSE,
-                             add.attributes = TRUE)
+                             output = "text", header = TRUE)
     print.lavaan.parameterEstimates(PE, ..., nd = nd)
 
     invisible(y)
