@@ -27,7 +27,7 @@ representation.LISREL <- function(partable = NULL,
     group.w.free  <- any(partable$lhs == "group" & partable$op == "%")
 
     # gamma?only if conditional.x
-    if(any(partable$op == "~" & partable$exo == 1L)) {
+    if(any(partable$op %in% c("~", "<~") & partable$exo == 1L)) {
         gamma <- TRUE
     } else {
         gamma <- FALSE
@@ -1124,6 +1124,38 @@ computeTHETA.LISREL <- function(MLIST=NULL,
     THETA
 }
 
+computeNU.LISREL <- function(MLIST=NULL,
+                             sample.mean = sample.mean,
+                             ov.y.dummy.ov.idx=NULL,
+                             ov.x.dummy.ov.idx=NULL,
+                             ov.y.dummy.lv.idx=NULL,
+                             ov.x.dummy.lv.idx=NULL) {
+
+    # get NU, but do not 'fix'
+    NU <- .internal_get_NU(MLIST = MLIST, sample.mean = sample.mean,
+                           ov.y.dummy.ov.idx = ov.y.dummy.ov.idx,
+                           ov.x.dummy.ov.idx = ov.x.dummy.ov.idx,
+                           ov.y.dummy.lv.idx = ov.y.dummy.lv.idx,
+                           ov.x.dummy.lv.idx = ov.x.dummy.lv.idx)
+
+    # ALPHA? (reconstruct, but no 'fix')
+    ALPHA <- .internal_get_ALPHA(MLIST = MLIST, sample.mean = sample.mean,
+                                 ov.y.dummy.ov.idx = ov.y.dummy.ov.idx,
+                                 ov.x.dummy.ov.idx = ov.x.dummy.ov.idx,
+                                 ov.y.dummy.lv.idx = ov.y.dummy.lv.idx,
+                                 ov.x.dummy.lv.idx = ov.x.dummy.lv.idx)
+
+    ov.dummy.idx = c(ov.y.dummy.ov.idx, ov.x.dummy.ov.idx)
+    lv.dummy.idx = c(ov.y.dummy.lv.idx, ov.x.dummy.lv.idx)
+
+    # fix NU
+    if(length(ov.dummy.idx) > 0L) {
+        NU[ov.dummy.idx, 1] <- ALPHA[lv.dummy.idx, 1]
+    }
+
+    NU
+}
+
 # compute IB.inv
 .internal_get_IB.inv <- function(MLIST = NULL) {
 
@@ -1504,7 +1536,7 @@ setResidualElements.LISREL <- function(MLIST=NULL,
     } else {
         delta <- MLIST$delta
     }
-    # theta = DELTA^(-1/2) - diag( LAMBDA (I-B)^-1 PSI (I-B)^-T t(LAMBDA) )
+    # theta = DELTA^(-2) - diag( LAMBDA (I-B)^-1 PSI (I-B)^-T t(LAMBDA) )
     RESIDUAL <- as.vector(1/(delta*delta) - diag.Sigma)
     if(length(num.idx) > 0L) {
         diag(MLIST$theta)[-num.idx] <- RESIDUAL[-num.idx]
