@@ -29,6 +29,27 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
         TEST$standard  <- TEST.unscaled
     }
 
+    # E.inv ok?
+    if( length(lavoptions$information) == 1L &&
+        length(lavoptions$h1.information) == 1L &&
+        length(lavoptions$observed.information) == 1L) {
+        E.inv.recompute <- FALSE
+    } else if( (lavoptions$information[1] == lavoptions$information[2]) &&
+        (lavoptions$h1.information[1] == lavoptions$h1.information[2]) &&
+        (lavoptions$observed.information[1] ==
+         lavoptions$observed.information[2]) ) {
+        E.inv.recompute <- FALSE
+    } else {
+        E.inv.recompute <- TRUE
+        # change information options
+        lavoptions$information[1] <- lavoptions$information[2]
+        lavoptions$h1.information[1] <- lavoptions$h1.information[2]
+        lavoptions$observed.information[1] <- lavoptions$observed.information[2]
+    }
+    if(!is.null(E.inv) && !is.null(WLS.V) && !is.null(Delta)) {
+        E.inv.recompute <- FALSE # user-provided
+    }
+
     # check test
     if(!all(test %in% c("satorra.bentler",
                         "scaled.shifted",
@@ -44,20 +65,20 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
     # check method
     if(method == "default") {
         method <- "ABA"
-    } else if(!all(method %in% c("original", "orthogonal.complement", 
+    } else if(!all(method %in% c("original", "orthogonal.complement",
                                  "ABA"))) {
-        warning("lavaan WARNING: method must be one of `original', `ABA', `orthogonal.complement'; will use `original'")
+        warning("lavaan WARNING: method must be one of `original', `ABA', `orthogonal.complement'; will use `ABA'")
         method <- "ABA"
     }
 
     # do we have E.inv, Delta, WLS.V?
-    if(is.null(E.inv) || is.null(Delta) || is.null(WLS.V)) {
+    if(is.null(E.inv) || is.null(Delta) || is.null(WLS.V) || E.inv.recompute) {
         if(mimic == "Mplus" && lavoptions$estimator == "ML") {
             E <- lav_model_information_expected_MLM(lavmodel = lavmodel,
                      augmented = FALSE, inverted = FALSE,
                      lavsamplestats=lavsamplestats, extra = TRUE)
         } else {
-            E <- lav_model_information(lavmodel = lavmodel, 
+            E <- lav_model_information(lavmodel = lavmodel,
                      lavimplied = lavimplied,
                      lavsamplestats = lavsamplestats, lavdata = lavdata,
                      lavoptions = lavoptions, extra = TRUE)
@@ -65,17 +86,35 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
         E.inv <- try(lav_model_information_augment_invert(lavmodel,
                          information = E, inverted = TRUE), silent=TRUE)
         if(inherits(E.inv, "try-error")) {
-            TEST <- list(test = test, stat = as.numeric(NA),
-                stat.group = rep(as.numeric(NA), lavsamplestats@ngroups),
-                df = TEST.unscaled$df, refdistr = TEST.unscaled$refdistr,
-                pvalue = as.numeric(NA), scaling.factor = as.numeric(NA))
-            warning("lavaan WARNING: could not invert information matrix\n")
-            return(TEST)
+
+            if(return.ugamma) {
+                warning("lavaan WARNING: could not invert information matrix needed for UGamma\n")
+                return(NULL)
+            } else if(return.u) {
+                warning("lavaan WARNING: could not invert information matrix needed for UfromUGamma\n")
+                return(NULL)
+            } else {
+                TEST[[test[1]]] <- c(TEST$standard,
+                                     scaling.factor = as.numeric(NA),
+                                     label = character(0))
+                warning("lavaan WARNING: could not invert information matrix needed for robust test statistic\n")
+                TEST[[test[1]]]$test <- test[1] # to prevent lavTestLRT error when robust test is detected for some but not all models
+                return(TEST)
+            }
         }
         Delta <- attr(E, "Delta")
         WLS.V <- attr(E, "WLS.V")
     }
 
+    # catch df == 0
+    if((TEST$standard$df == 0L || TEST$standard$df < 0) &&
+       !return.u && !return.ugamma) {
+        TEST[[test[1]]] <- c(TEST$standard, scaling.factor = as.numeric(NA),
+                             label = character(0))
+        TEST[[test[1]]]$test <- test[1] # to prevent lavTestLRT error when robust test is detected for some but not all models
+        return(TEST)
+    }
+
     # Gamma
     if(is.null(Gamma)) {
         Gamma <- lavsamplestats@NACOV
@@ -99,8 +138,8 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
 
     if(method == "original") {
         out <- lav_test_satorra_bentler_trace_original(Gamma = Gamma,
-                   Delta = Delta, WLS.V = WLS.V, E.inv = E.inv, 
-                   ngroups = ngroups, nobs = lavsamplestats@nobs, 
+                   Delta = Delta, WLS.V = WLS.V, E.inv = E.inv,
+                   ngroups = ngroups, nobs = lavsamplestats@nobs,
                    ntotal = lavsamplestats@ntotal, return.u = return.u,
                    return.ugamma = return.ugamma,
                    Satterthwaite = Satterthwaite)
@@ -124,7 +163,6 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
     trace.UGamma  <- out$trace.UGamma
     trace.UGamma2 <- out$trace.UGamma2
 
-
     if("satorra.bentler" %in% test) {
         # same df
         df.scaled <- TEST$standard$df
@@ -139,14 +177,31 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
         # scaled test statistic global
         stat <- sum(stat.group)
 
+        # label
+        if(mimic == "Mplus") {
+            if(lavoptions$estimator == "ML") {
+                label <-
+                    "Satorra-Bentler correction (Mplus variant)"
+            } else if(lavoptions$estimator == "DWLS") {
+                label <-
+                    "Satorra-Bentler correction (WLSM)"
+            } else if(lavoptions$estimator == "ULS") {
+                label <-
+                    "Satorra-Bentler correction (ULSM)"
+            }
+        } else {
+            label <- "Satorra-Bentler correction"
+        }
+
         TEST$satorra.bentler <-
-            list(test            = "satorra.bentler",
-                 stat            = stat,
-                 stat.group      = stat.group,
-                 df              = df.scaled,
-                 pvalue          = 1 - pchisq(stat, df.scaled),
-                 trace.UGamma    = trace.UGamma,
-                 scaling.factor  = scaling.factor)
+            list(test                 = "satorra.bentler",
+                 stat                 = stat,
+                 stat.group           = stat.group,
+                 df                   = df.scaled,
+                 pvalue               = 1 - pchisq(stat, df.scaled),
+                 trace.UGamma         = trace.UGamma,
+                 scaling.factor       = scaling.factor,
+                 label = label)
     }
 
     if("mean.var.adjusted" %in% test) {
@@ -167,15 +222,32 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
         # scaled test statistic global
         stat <- sum(stat.group)
 
+        # label
+        if(mimic == "Mplus") {
+            if(lavoptions$estimator == "ML") {
+                label <-
+                    "mean and variance adjusted correction (MLMV)"
+            } else if(lavoptions$estimator == "DWLS") {
+                label <-
+                    "mean and variance adjusted correction (WLSMV)"
+            } else if(lavoptions$estimator == "ULS") {
+                label <-
+                    "mean and variance adjusted correction (ULSMV)"
+            }
+        } else {
+            label <- "mean and variance adjusted correction"
+        }
+
         TEST$mean.var.adjusted <-
-            list(test            = "mean.var.adjusted",
-                 stat            = stat,
-                 stat.group      = stat.group,
-                 df              = df.scaled,
-                 pvalue          = 1 - pchisq(stat, df.scaled),
-                 trace.UGamma    = trace.UGamma,
-                 trace.UGamma2   = trace.UGamma2,
-                 scaling.factor  = scaling.factor)
+            list(test                 = "mean.var.adjusted",
+                 stat                 = stat,
+                 stat.group           = stat.group,
+                 df                   = df.scaled,
+                 pvalue               = 1 - pchisq(stat, df.scaled),
+                 trace.UGamma         = trace.UGamma,
+                 trace.UGamma2        = trace.UGamma2,
+                 scaling.factor       = scaling.factor,
+                 label = label)
     }
 
     if("scaled.shifted" %in% test) {
@@ -204,17 +276,34 @@ lav_test_satorra_bentler <- function(lavobject      = NULL,
 
         # scaled test statistic global
         stat <- sum(stat.group)
-        
+
+        # label
+        if(mimic == "Mplus") {
+            if(lavoptions$estimator == "ML") {
+                label <-
+                    "simple second-order correction (MLMV)"
+            } else if(lavoptions$estimator == "DWLS") {
+                label <-
+                    "simple second-order correction (WLSMV)"
+            } else if(lavoptions$estimator == "ULS") {
+                label <-
+                    "simple second-order correction (ULSMV)"
+            }
+        } else {
+            label <- "simple second-order correction"
+        }
+
         TEST$scaled.shifted <-
-            list(test            = "scaled.shifted",
-                 stat            = stat,
-                 stat.group      = stat.group,
-                 df              = df.scaled,
-                 pvalue          = 1 - pchisq(stat, df.scaled),
-                 trace.UGamma    = trace.UGamma,
-                 trace.UGamma2   = trace.UGamma2,
-                 scaling.factor  = scaling.factor,
-                 shift.parameter = shift.parameter)
+            list(test                 = "scaled.shifted",
+                 stat                 = stat,
+                 stat.group           = stat.group,
+                 df                   = df.scaled,
+                 pvalue               = 1 - pchisq(stat, df.scaled),
+                 trace.UGamma         = trace.UGamma,
+                 trace.UGamma2        = trace.UGamma2,
+                 scaling.factor       = scaling.factor,
+                 label                = label,
+                 shift.parameter      = shift.parameter)
     }
 
     if(return.ugamma) {
@@ -240,7 +329,7 @@ lav_test_satorra_bentler_trace_original <- function(Gamma         = NULL,
                                                     return.u      = FALSE,
                                                     return.ugamma = FALSE,
                                                     Satterthwaite = FALSE) {
- 
+
     # trace of UGamma per group
     trace.UGamma  <- trace.UGamma2 <- rep(as.numeric(NA), ngroups)
 
@@ -260,8 +349,8 @@ lav_test_satorra_bentler_trace_original <- function(Gamma         = NULL,
             WLS.Vg <- diag(WLS.Vg)
         }
 
-        U <- (WLS.Vg - WLS.Vg %*% Delta[[g]] %*% E.inv %*% 
-                                t(Delta[[g]]) %*% WLS.Vg) 
+        U <- (WLS.Vg - WLS.Vg %*% Delta[[g]] %*% E.inv %*%
+                                t(Delta[[g]]) %*% WLS.Vg)
         trace.UGamma[g] <- sum(U * Gamma.g)
 
         if(return.u) {
@@ -279,7 +368,7 @@ lav_test_satorra_bentler_trace_original <- function(Gamma         = NULL,
     trace.UGamma <- sum(trace.UGamma)
     trace.UGamma2 <- sum(trace.UGamma2)
 
-    list(trace.UGamma = trace.UGamma, trace.UGamma2 = trace.UGamma2, 
+    list(trace.UGamma = trace.UGamma, trace.UGamma2 = trace.UGamma2,
          UGamma = UG, UfromUGamma = UfromUGamma)
 }
 
@@ -312,6 +401,7 @@ lav_test_satorra_bentler_trace_complement <- function(Gamma         = NULL,
         }
 
         # handle equality constraints
+        # FIXME: inequality constraints are ignored!
         if(lavmodel@eq.constraints) {
             Delta.g <- Delta.g %*% lavmodel@eq.constraints.K
         }
@@ -352,7 +442,7 @@ lav_test_satorra_bentler_trace_complement <- function(Gamma         = NULL,
 #
 #    = A1.inv %*% AGA1 -
 #      A1.inv %*% AGA1 %*% Delta %*% E.inv %*% tDelta %*% A1
-#    
+#
 # if only the trace is needed, we can use reduce the rhs (after the minus)
 # to AGA1 %*% Delta %*% E.inv %*% tDelta (eliminating A1 and A1.inv)
 
@@ -391,10 +481,10 @@ lav_test_satorra_bentler_trace_ABA <- function(Gamma         = NULL,
         # note: we have AGA1 at the end, to avoid ending up with
         # a transposed matrix (both parts are non-symmetric)
         if(diagonal) {
-            UG <- t(Gamma.g * a1) - 
+            UG <- t(Gamma.g * a1) -
                   (Delta.g %*% tcrossprod(E.inv, Delta.g) %*% AGA1)
         } else {
-            UG <- (Gamma.g %*% A1) - 
+            UG <- (Gamma.g %*% A1) -
                   (Delta.g %*% tcrossprod(E.inv, Delta.g) %*% AGA1)
         }
 
